/**
 * @file	trianglemesh.h
 * @author	Andrew Robert Owens
 * @date 	2018-2-24.
 * @brief	CollisionExample: trianglemesh.h
 * @details Organization: Biological Modeling and Visualization research group
 * University of Calgary, Calgary, AB, Canada
 *
 * Contact:	arowens [at] ucalgary.ca
 * @copyright Copyright (c) 2018 ___ALGORITHMIC_BOTANY___. All rights reserved.
 *
 * @brief
 */

#pragma once

#include <vector>

#include "triangle.h"
#include "vec3f.h"

namespace geometry {

class TriangleMesh {
public:
  TriangleMesh() = default;
  TriangleMesh(std::vector<math::Vec3f> vertices,
               std::vector<Triangle<int>> indexTriangles);

  void reloadVertices(std::vector<math::Vec3f> vertices);

  std::vector<math::Vec3f> const &vertices() const;

  std::vector<geometry::Triangle<int>> const &triangles() const;

  unsigned int vertexCount() const;

  unsigned int triangleCount() const;

  geometry::Triangle<int> const *triangleData() const;

  math::Vec3f const *vertexData() const;

private:
  std::vector<math::Vec3f> m_vertices;
  std::vector<geometry::Triangle<int>> m_indexTriangles;
};

std::vector<math::Vec3f> calculateTriangleNormals(TriangleMesh const &mesh);

std::vector<math::Vec3f> calculateTriangleNormals(
    std::vector<math::Vec3f> const &vertices,
    std::vector<geometry::Triangle<int>> const &indexTriangles);

std::vector<math::Vec3f>
calculateVertexNormals(TriangleMesh const &mesh,
                       std::vector<math::Vec3f> const &triangleNormals);

std::vector<math::Vec3f> calculateVertexNormals(
    std::vector<math::Vec3f> const &vertices,
    std::vector<geometry::Triangle<int>> const &indexTriangles,
    std::vector<math::Vec3f> const &triangleNormals);

} // namespace geometry
