//
//  utility.h
//  CollisionExample
//
//  Created by Andrew Owens on 2018-02-21.
//  Copyright (c) 2018 ___ALGORITHMIC_BOTANY___. All rights reserved.
//

#pragma once

#include <type_traits>

namespace math {

/**
 * Type-safe and precision appropriate conversion from radians to degrees
 * TODO: variable template C++14
 */
template <typename Float> constexpr Float radianToDegree(Float radian);

/**
 * Type-safe and precision appropriate conversion from degrees to radians
 * TODO: variable template C++14
 */
template <typename Float> constexpr Float degreeToRadian(Float degree);

/**
 * Compares floating point (float, double, long double) to zero to within a
 * default epsilon (std::numeric_limits<Float>::epsilon())
 */
template <typename Float> constexpr bool isZero(Float x);

/**
 * Compares floating point (float, double, long double) to zero to within a user
 * defined epsilon
 */
template <typename Float> constexpr bool isZero(Float x, Float epsilon);

/**
 * Return the sign of the value: sgn(val)
 */
template <typename Signed> constexpr int sgn(Signed val);

/**
 * Returns the absolute value of a signed value
 */
template <typename Signed> constexpr Signed abs(const Signed &value);

/**
 * Retruns minimum of two arguments, returning the first argument upon
 * equivalence.
 */
template <typename Type>
constexpr Type const &minimum(Type const &a, Type const &b);

/**
 * Retruns minimum of two arguments using a custom comparitor, returning the
 * first argument upon equivalence.
 */
template <typename Type, typename Comp>
constexpr Type const &minimum(Type const &a, Type const &b, Comp comp);

/**
 * Retruns minimum of a variadic argument list, returning the first argument
 * upon equivalence.
 */
template <typename T, typename... Args>
constexpr T const &minimum(T const &a, T const &b, Args... args);

/**
 * Retruns minimum of a variadic argument list using a custom comparitor,
 * returning the first argument upon equivalence.
 */
template <typename T, typename... Args, typename Comp>
constexpr T const &minimum(T const &a, T const &b, Args... args, Comp comp);

/**
 * Retruns maximum of two arguments, returning the second argument upon
 * equivalence.
 */
template <typename Type>
constexpr Type const &maximum(Type const &a, Type const &b);

/**
 * Retruns maximum of two arguments using a custom comparitor, returning the
 * second argument upon equivalence.
 */
template <typename Type, typename Comp>
constexpr Type const &maximum(Type const &a, Type const &b, Comp comp);

/**
 * Retruns maximum of a variadic argument list, returning the second argument
 * upon equivalence.
 */
template <typename T, typename... Args>
constexpr T const &maximum(T const &a, T const &b, Args... args);

/**
 * Retruns maximum of a variadic argument list using a custom comparitor,
 * returning the second argument upon equivalence.
 */
template <typename T, typename... Args, typename Comp>
constexpr T const &maximum(T const &a, T const &b, Args... args, Comp comp);

/**
 * Clamp generic type
 */
template <class T>
constexpr T const &clamp(const T &v, const T &lo, const T &hi);

/**
 * Clamp generic types with custum comparitor
 */
template <class T, class Compare>
constexpr T const &clamp(const T &v, const T &lo, const T &hi, Compare comp);

} // namespace math

#include "math_utility.tpp"
