#pragma once

#include <iosfwd>

namespace math {

// struct declaration
struct Vec3f {
  union {
    struct {
      float m_x;
      float m_y;
      float m_z;
    };
    //    float m_coord[3] = {0.f, 0.f, 0.f}; // TODO: clang seems to force
    //    this?
    float m_coord[3];
  };

  // Vec3f() = default; // TODO: clang issue from above
  Vec3f();
  Vec3f(float x, float y, float z);

  /*
   * Mutating member functions
   */
  Vec3f &operator+=(Vec3f const &rhs);
  Vec3f &operator-=(Vec3f const &rhs);
  Vec3f &operator*=(float rhs);
  Vec3f &operator/=(float rhs);

  Vec3f &operator=(Vec3f const &rhs) = default;

  float &operator[](size_t id);
  float const &operator[](size_t id) const;

  float x() const;
  float y() const;
  float z() const;

  float *data();
  float const *data() const;

  Vec3f &normalize();

  void zero();
};

// Free function declarations
Vec3f operator+(Vec3f const &a, Vec3f const &b);
Vec3f operator-(Vec3f const &a, Vec3f const &b);
Vec3f operator*(float s, Vec3f v);
Vec3f operator*(Vec3f v, float s);
Vec3f operator/(Vec3f v, float s);
Vec3f operator-(Vec3f v);

float operator*(Vec3f const &a, Vec3f const &b);
float dot(Vec3f const &a, Vec3f const &b);

Vec3f operator^(Vec3f const &a, Vec3f const &b);
Vec3f cross(Vec3f const &a, Vec3f const &b);

float norm(Vec3f const &v);
float normSquared(Vec3f const &v);
Vec3f normalized(Vec3f v);

// Linear interpolation from a to b by t
Vec3f lerp(Vec3f const &a, Vec3f const &b, float t);
float distance(Vec3f const &a, Vec3f const &b);
float distanceSquared(Vec3f const &a, Vec3f const &b);

Vec3f rotateAroundAxis(Vec3f v, Vec3f axis, float angleDegrees);
Vec3f rotateAroundNormalizedAxis(Vec3f v, Vec3f const &axis,
                                 float angleDegrees);

Vec3f componentMultiplication(Vec3f const &lhs, Vec3f const &rhs);

Vec3f projectOnto(Vec3f inputVector, Vec3f const &axis);

Vec3f orthogonalize(Vec3f inputVector, Vec3f const &axis);

std::istream &operator>>(std::istream &in, Vec3f &v);
std::ostream &operator<<(std::ostream &out, Vec3f const &v);

} // namespace math
