/**
 * @file	buffer_object.h
 * @author	Andrew Robert Owens
 * @date 	2018-2-26.
 * @brief	CollisionExample: buffer_object.h
 * @details Organization: Biological Modeling and Visualization research group
 * University of Calgary, Calgary, AB, Canada
 *
 * Contact:	arowens [at] ucalgary.ca
 * @copyright Copyright (c) 2018 ___ALGORITHMIC_BOTANY___. All rights reserved.
 *
 * @brief
 */

#pragma once

#include <glad/glad.h>

namespace openGL {

class BufferObject {
public:
  enum Type : GLenum {
    INVALID = GL_INVALID_ENUM,
    ARRAY = GL_ARRAY_BUFFER,
    ELEMENT_ARRAY = GL_ELEMENT_ARRAY_BUFFER,
    TEXTURE = GL_TEXTURE_BUFFER
  };

  enum : GLuint { INVALID_ID = 0 };

public:
  ~BufferObject();

  /* Explicitly remove copy/assignment (RAII) */
  BufferObject(BufferObject const &) = delete;
  BufferObject &operator=(BufferObject const &) = delete;

  /* Retain move semantics */
  BufferObject(BufferObject &&other);
  BufferObject &operator=(BufferObject &&other);

  BufferObject::Type type() const;
  bool isAllocated() const;
  bool isValidType() const;
  bool isValid() const;

  void bind() const;

  void bind(Type type);

private:
  /* Only called through makeBufferObject() factory function */
  BufferObject(GLuint id);
  void release();

  friend BufferObject makeBufferObject();
  friend void swap(BufferObject &lhs, BufferObject &rhs);

private:
  GLuint m_id = INVALID_ID;
  BufferObject::Type m_type = INVALID;
};

BufferObject makeBufferObject();
void swap(BufferObject &lhs, BufferObject &rhs);

bool isValidBufferObjectID(GLuint id);

bool isValidBufferObjectType(BufferObject::Type type);

BufferObject::Type enumToBufferObjectType(GLenum bufferObjectEnum);

} // namespace openGL
