#pragma once

#include <string>

#include <glad/glad.h>

namespace openGL {

class Shader {
public:
  enum Type : GLenum {
    INVALID = GL_INVALID_ENUM,
    VERTEX = GL_VERTEX_SHADER,
    GEOMETRY = GL_GEOMETRY_SHADER,
    FRAGMENT = GL_FRAGMENT_SHADER
  };

  enum : GLuint { INVALID_ID = 0 };

  ~Shader();

  /* Explicitly remove copy/assignment (RAII) */
  Shader(Shader const &) = delete;
  Shader &operator=(Shader const &) = delete;

  /* Retain move semantics */
  Shader(Shader &&other);

  Shader &operator=(Shader &&other);

  bool isValid() const;

  Shader::Type type() const;

  GLuint id() const;

private:
  /* Only called through makeShader() factory function */
  Shader(GLuint id, Shader::Type shaderType);

  void release();

  friend Shader makeShader(std::string const &shaderCode,
                           Shader::Type shaderType);
  friend void swap(Shader &lhs, Shader &rhs);

private:
  GLuint m_id = INVALID_ID;
  Shader::Type m_shaderType = INVALID;
};

Shader makeShader(std::string const &shaderCode, GLenum shaderType);

void swap(Shader &lhs, Shader &rhs);

Shader::Type enumToShaderType(GLenum shaderTypeEnum);

bool checkCompileStatus(GLuint shaderID);

bool isValidShaderID(GLuint id);

bool isValidShaderType(Shader::Type type);

} // namespace openGL
