/**
 * @file	vertex_array_object.h
 * @author	Andrew Robert Owens
 * @date 	2018-2-26.
 * @brief	CollisionExample: vertex_array_object.h
 * @details Organization: Biological Modeling and Visualization research group
 * University of Calgary, Calgary, AB, Canada
 *
 * Contact:	arowens [at] ucalgary.ca
 * @copyright Copyright (c) 2018 ___ALGORITHMIC_BOTANY___. All rights reserved.
 *
 * @brief
 */

#pragma once

#include <glad/glad.h>

namespace openGL {

class VertexArrayObject {
public:
  enum : GLuint { INVALID_ID = 0 };

public:
  ~VertexArrayObject();

  /* Explicitly remove copy/assignment (RAII) */
  VertexArrayObject(VertexArrayObject const &) = delete;
  VertexArrayObject &operator=(VertexArrayObject const &) = delete;

  /* Retain move semantics */
  VertexArrayObject(VertexArrayObject &&other);
  VertexArrayObject &operator=(VertexArrayObject &&other);

  bool isValid() const;
  void bind() const;

private:
  /* Only called through makeVertexAttributeObject() factory function */
  VertexArrayObject(GLuint id);
  void release();

  friend VertexArrayObject makeVertexArrayObject();
  friend void swap(VertexArrayObject &lhs, VertexArrayObject &rhs);

private:
  GLuint m_id = INVALID_ID;
};

bool isValid_VAO_ID(GLuint id);

VertexArrayObject makeVertexArrayObject();
void swap(VertexArrayObject &lhs, VertexArrayObject &rhs);

} // namespace openGL
