/**
 * @file	triangle_mesh_file_io.cpp
 * @author	Andrew Robert Owens
 * @date 	2018-2-27.
 * @brief	CollisionExample: triangle_mesh_file_io.cpp
 * @details Organization: Biological Modeling and Visualization research group
 * University of Calgary, Calgary, AB, Canada
 *
 * Contact:	arowens [at] ucalgary.ca
 * @copyright Copyright (c) 2018 ___ALGORITHMIC_BOTANY___. All rights reserved.
 *
 * @brief
 */

#include "triangle_mesh_file_io.h"

#include <array>
#include <fstream>
#include <iostream>
#include <iterator>
#include <sstream>
#include <stdexcept>

#include "vec3f.h"

namespace geometry {

nonstd::optional<geometry::TriangleMesh>
loadTriangleMeshFromFile(std::string const &filePath) {

  std::ifstream in(filePath.c_str());

  if (!in.is_open()) {
    std::cerr << "Could not open file\n";
    return nonstd::nullopt;
  }

  std::vector<math::Vec3f> vertices;
  std::vector<geometry::Triangle<int>> triangles;

  std::string s;

  math::Vec3f tmp3f;
  // Vec2f tmp2f;
  std::string tmpStr;

  while (getline(in, s)) {
    std::stringstream line(s);
    std::string token;

    line >> token;

    if (token == "v") {
      line >> tmp3f;
      vertices.push_back(tmp3f);
    } else if (token == "vt") {
      /*
      line >> tmp2f;
      m_uvs.push_back(tmp2f);
      // currently ignore;
      */
    } else if (token == "g") // push group
    {
      /*
      line >> tmpStr;
      Group gNew(tmpStr);

      if (hasGroup()) {
        Group &gOld = currentGroup();
        gOld.facesEnd = m_faces.size();
      }

      gNew.facesStart = m_faces.size();
      addGroup(gNew);
      */
    } else if (token == "o") // obj name
    {
      /*
      line >> tmpStr;
      m_name = tmpStr;
      */
    } else if (token == "f") {
      typedef std::array<int, 3> Indices;
      Indices indices = {-1, -1, -1}; // init

      // hopefully CCW winding
      std::vector<Indices> verts;
      while (line >> s) {
        std::stringstream item(s);

        // reset
        indices[0] = -1;
        indices[1] = -1;
        indices[2] = -1;

        for (int j = 0; getline(item, s, '/') && j < 3; ++j) {
          std::stringstream index(s);
          index >> indices[j];
        }

        // Indices are 0 based, so minus by 1
        if (indices[0] != -1) {
          indices[0] -= 1;
        }

        if (indices[1] != -1) {
          indices[1] -= 1;
        }

        if (indices[2] != -1) {
          indices[2] -= 1;
        }

        verts.push_back(indices); // polygon vert
      }

      if (verts.size() == 3) // triangle
      {
        geometry::Triangle<int> triangle;
        for (int i = 0; i < 3; ++i) {
          triangle.vertex[i] = verts[i][0];
          // triangle.uvIndex[i] = verts[i][1];
          //					triangle.normal[i] =
          // verts[i][2];
        }
        triangles.push_back(triangle);
      } else if (verts.size() == 4) // Quad
      {

        geometry::Triangle<int> triangle;
        for (int i = 0; i < 3; ++i) {
          triangle.vertex[i] = verts[i][0];
          // triangle.uvIndex[i] = verts[i][1];
          //					triangle.normal[i] =
          // verts[i][2];
        }
        triangles.push_back(triangle);

        int j = 2;
        for (int i = 0; i < 3; ++i) {
          triangle.vertex[i] = verts[j][0];
          // triangle.uvIndex[i] = verts[i][1];
          //					triangle.normal[i] =
          // verts[j][2];

          j = (j + 1) % 4;
        }
        triangles.push_back(triangle);

      } else {
        std::cerr << "invalid obj\n";
        return nonstd::nullopt;
      }
    }
  }

  return TriangleMesh{vertices, triangles};
}

} // namespace geometry
