#include "polynomial_roots.h"

#include <cmath>

#include "math_utility.h"

namespace math {

int solveQuadraticRoots(float c[3], float r[2], float const EPSILON) {
  return solveQuadraticRoots(c[2], c[1], c[0], r, EPSILON);
}

int solveQuadraticRoots(float const &a, float const &b, float const &c,
                        float r[2], float const EPSILON) {
  using std::sqrt;

  float p, q, D;

  if (isZero(a, EPSILON)) {
    return solveLinearRoots(b, c, r, EPSILON);
  }

  p = b / (float(2) * a);
  q = c / a;
  D = p * p - q;

  if (isZero(D, EPSILON)) {
    // one root
    r[0] = -p;
    return 1;
  }

  if (D < float(0)) {
    // no real roots
    return 0;
  } else {
    // two real roots
    float sqrt_D = sqrt(D);
    r[0] = sqrt_D - p;
    r[1] = -sqrt_D - p;
    return 2;
  }
}

int solveLinearRoots(float c[2], float r[1], float const EPSILON) {
  return solveLinearRoots(c[1], c[0], r, EPSILON);
}

int solveLinearRoots(float const &a, float const &b, float r[1],
                     float const EPSILON) {
  if (isZero(a, EPSILON)) // constant function
  {
    if (isZero(b, EPSILON)) {
      return -1; // infinite solutions
    }
    return 0; // no solutions
  }

  r[0] = -b / a;
  return 1;
}

} // namespace math
