#include "vec4f.h"

#include <cmath>
#include <iostream>

namespace math {

// Member function interface
// TODO added to appese clang: initialized {0,0,0} bug
Vec4f::Vec4f() : m_x(0.f), m_y(0.f), m_z(0.f), m_w(0.f) {}

Vec4f::Vec4f(float x, float y, float z, float w)
    : m_x(x), m_y(y), m_z(z), m_w(w) {}

Vec4f::operator Vec3f() { return {m_x, m_y, m_z}; }

Vec4f &Vec4f::operator+=(Vec4f const &rhs) {
  m_x += rhs.m_x;
  m_y += rhs.m_y;
  m_z += rhs.m_z;
  m_w += rhs.m_z;
  return *this;
}
Vec4f &Vec4f::operator-=(Vec4f const &rhs) {
  m_x -= rhs.m_x;
  m_y -= rhs.m_y;
  m_z -= rhs.m_z;
  m_w -= rhs.m_z;
  return *this;
}

Vec4f &Vec4f::operator*=(float rhs) {
  m_x *= rhs;
  m_y *= rhs;
  m_z *= rhs;
  m_w *= rhs;
  return *this;
}

Vec4f &Vec4f::operator/=(float rhs) {
  m_x /= rhs;
  m_y /= rhs;
  m_z /= rhs;
  m_w /= rhs;
  return *this;
}

float &Vec4f::operator[](size_t id) { return m_coord[id]; }

float const &Vec4f::operator[](size_t id) const { return m_coord[id]; }

float Vec4f::x() const { return m_x; }

float Vec4f::y() const { return m_y; }

float Vec4f::z() const { return m_z; }

float Vec4f::w() const { return m_w; }

float *Vec4f::data() { return &m_coord[0]; }

float const *Vec4f::data() const { return &m_coord[0]; }

Vec4f &Vec4f::normalize() {
  float l = norm(*this);
  return (*this) /= l;
}

void Vec4f::zero() {
  m_x = 0.f;
  m_y = 0.f;
  m_z = 0.f;
  m_w = 0.f;
}

// Free function
/*
 * Vector-Vector Addition/ Subtraction
 */
Vec4f operator+(Vec4f const &a, Vec4f const &b) {
  return Vec4f(a.m_x + b.m_x, //
               a.m_y + b.m_y, //
               a.m_z + b.m_z, //
               a.m_w + b.m_w);
}
Vec4f operator-(Vec4f const &a, Vec4f const &b) {
  return Vec4f(a.m_x - b.m_x, //
               a.m_y - b.m_y, //
               a.m_z - b.m_z, //
               a.m_w - b.m_w);
}

/*
 * Scalar-Vector Multiplication/Division
 */
Vec4f operator*(float s, Vec4f v) {
  v.m_x *= s;
  v.m_y *= s;
  v.m_z *= s;
  v.m_w *= s;
  return v;
}
Vec4f operator*(Vec4f v, float s) { return s * v; }

Vec4f operator/(Vec4f v, float s) {
  v.m_x /= s;
  v.m_y /= s;
  v.m_z /= s;
  v.m_w /= s;
  return v;
}

/*
 * Negation of vector
 * -v = (-1.f) * v
 */
Vec4f operator-(Vec4f v) {
  v.m_x = -v.m_x;
  v.m_y = -v.m_y;
  v.m_z = -v.m_z;
  v.m_x = -v.m_w;
  return v;
}

/*
 * Vector norm (length)
 */
float norm(Vec4f const &v) {
  return std::sqrt(v.m_x * v.m_x + v.m_y * v.m_y + v.m_z * v.m_z);
}
float normSquared(Vec4f const &v) {
  return v.m_x * v.m_x + v.m_y * v.m_y + v.m_z * v.m_z;
}

/*
 * Normalized Vector
 */
Vec4f normalized(Vec4f v) {
  float l = norm(v);
  return v /= l;
}

std::ostream &operator<<(std::ostream &out, Vec4f const &v) {
  return out << v.m_x << " " << v.m_y << " " << v.m_z;
}

std::istream &operator>>(std::istream &in, Vec4f &v) {
  return in >> v.m_x >> v.m_y >> v.m_z;
}

} // namespace math
