/**
 * @file	vertex_array_object.cpp
 * @author	Andrew Robert Owens
 * @date 	2018-2-26.
 * @brief	CollisionExample: vertex_array_object.cpp
 * @details Organization: Biological Modeling and Visualization research group
 * University of Calgary, Calgary, AB, Canada
 *
 * Contact:	arowens [at] ucalgary.ca
 * @copyright Copyright (c) 2018 ___ALGORITHMIC_BOTANY___. All rights reserved.
 *
 * @brief
 */
#include "vertex_array_object.h"

#include <utility>

namespace openGL {

VertexArrayObject::VertexArrayObject(GLuint id) : m_id(id) {}

VertexArrayObject::~VertexArrayObject() { release(); }

VertexArrayObject::VertexArrayObject(VertexArrayObject &&other)
    : m_id(other.m_id) {
  other.m_id = INVALID_ID;
}

VertexArrayObject &VertexArrayObject::operator=(VertexArrayObject &&other) {
  if (this != &other) {
    release();
    swap(*this, other);
  }

  return *this;
}

void VertexArrayObject::release() {
  glDeleteVertexArrays(1, &m_id);
  m_id = INVALID_ID;
}

VertexArrayObject makeVertexArrayObject() {
  GLuint id = VertexArrayObject::INVALID_ID;

  glGenVertexArrays(1, &id);

  if (!isValid_VAO_ID(id))
    return VertexArrayObject(VertexArrayObject::INVALID_ID);

  return VertexArrayObject(id);
}

bool VertexArrayObject::isValid() const { return isValid_VAO_ID(m_id); }

void VertexArrayObject::bind() const { glBindVertexArray(m_id); }

bool isValid_VAO_ID(GLuint id) { return id != VertexArrayObject::INVALID_ID; }

void swap(VertexArrayObject &lhs, VertexArrayObject &rhs) {
  std::swap(lhs.m_id, rhs.m_id);
}

} // namespace openGL
